require "util"
--require "defines"

local PollutionWeight   = 5.0
local SpawnerWeight     = 3.0
local TimeWeight        = 2.0
local TotalWeight       = PollutionWeight + SpawnerWeight + TimeWeight

local PollutionDivisor  = 30 * 1000 * 1000 -- 30 Million.
local TimeCap = 24 * 60 * 60 * 60 -- 24 Hours
local SpawnerCap = 120 -- 120 Spawner deaths for maximum evolution
local SpawnerDecay = 1/(60*12) -- 1 per 12 minutes, 1/3600


script.on_init(function()
   global.pollution_spawner = 0
   game.map_settings.enemy_evolution.time_factor = 0.000008 * 0
   game.map_settings.enemy_evolution.pollution_factor = 0.00003 * 0
end)

script.on_configuration_changed(function(data)
	if data.mod_changes ~= nil and data.mod_changes["Rescaled-Evolution-Factor"] ~= nil and data.mod_changes["Rescaled-Evolution-Factor"].old_version == nil then
		global.pollution_spawner = 0
		game.map_settings.enemy_evolution.time_factor = 0.000008 * 0
		game.map_settings.enemy_evolution.pollution_factor = 0.00003 * 0
	end
 end)
 
 script.on_load(function()
	if remote.interfaces.EvoGUI ~= nil then registriere_sensoren() end
 end)

function pollutiontest(event)
    if game.tick % 60 == 24 then
		if remote.interfaces.EvoGUI ~= nil then registriere_sensoren() end

		local summe = 0 -- Pollution Count
		local anzahl = 0 -- ChunkCount
      
		local s = game.surfaces["nauvis"]
	  
		for c in s.get_chunks() do
			summe = summe + s.get_pollution({c.x *32,c.y *32})
			anzahl = anzahl + 1
		end
	  
		if anzahl < 1000 then --Mindestwert für die anzahl, erleichtert den Start.
			anzahl = 1000     --Minimum value for the number , facilitates the start
		end                       --Until this many chunks are loaded, assume you have at least 1000. Past that, reduces threshhold.
	  
		local pollution_summe = math.pow((summe/math.log(anzahl))/PollutionDivisor,0.5) * PollutionWeight --(Weight = 50%)
	  
		if pollution_summe > PollutionWeight then
			pollution_summe = PollutionWeight
		end
	  
		local vergangene_zeit = TimeWeight
	  
		if game.tick < TimeCap then --5184000 Ticks = 24 Hours
			vergangene_zeit = math.pow(game.tick / TimeCap, 2) * TimeWeight --(Weight = 20%)
		end
	  
		local pollution_spawner_anzahl = SpawnerWeight
		if global.pollution_spawner < SpawnerCap then
			pollution_spawner_anzahl = (global.pollution_spawner / SpawnerCap) * SpawnerWeight -- /60 * 3 (Weight = 30%)
		end
	  
		local factor = (pollution_summe + pollution_spawner_anzahl + vergangene_zeit) / TotalWeight --Again "entwichten". Untranslatable.
	  
		if factor > 1 then
			factor = 1
		end
	  
		game.evolution_factor = factor	  
		
		if remote.interfaces.EvoGUI ~= nil then
			remote.call("EvoGUI", "update_remote_sensor", "rescaled_evolution_factor_sensor_pollution", {"", {"rescaled_evolution_factor_sensor_pollution_prefix"}, " ", summe_format(summe)})
			remote.call("EvoGUI", "update_remote_sensor", "rescaled_evolution_factor_sensor_spawner", {"", {"rescaled_evolution_factor_sensor_spawner_prefix"}, " ", math.ceil(global.pollution_spawner)})
			remote.call("EvoGUI", "update_remote_sensor", "rescaled_evolution_factor_sensor_anteile", {"", {"rescaled_evolution_factor_sensor_anteile_prefix"}, " ", string.format("%.1f", vergangene_zeit * 10).."%|", string.format("%.1f", pollution_spawner_anzahl * 10).."%|", string.format("%.1f", pollution_summe * 10).."%"})
		end
	end
   
    if (game.tick % 60 == 25 and global.pollution_spawner > 0) then
		global.pollution_spawner = global.pollution_spawner -SpawnerDecay   --Vergiss mit der Zeit alte Fehden.
                                                                    --Forget the time old feuds .
	end
end

function spawner_death(event)
   if (global.pollution_spawner < SpawnerCap) and (event.entity.type == "unit-spawner") then
      global.pollution_spawner = global.pollution_spawner + 1

   end   
end

function registriere_sensoren()
	if global.ist_registriert == nil then
		local sensor_data = {mod_name = "Evolve Factor",
							 name = "rescaled_evolution_factor_sensor_pollution",
							 text = {"rescaled_evolution_factor_sensor_pollution_display", 42},
							 caption = {"rescaled_evolution_factor_sensor_pollution_caption"}}
   
		remote.call("EvoGUI", "create_remote_sensor", sensor_data)
   
		sensor_data = {mod_name = "Evolve Factor",
					   name = "rescaled_evolution_factor_sensor_spawner",
					   text = {"rescaled_evolution_factor_sensor_spawner_display", 42},
					  caption = {"rescaled_evolution_factor_sensor_spawner_caption"}}
   
		remote.call("EvoGUI", "create_remote_sensor", sensor_data)
   
		sensor_data = {mod_name = "Evolve Factor",
					   name = "rescaled_evolution_factor_sensor_anteile",
					   text = {"rescaled_evolution_factor_sensor_anteile_display", 42},
					   caption = {"rescaled_evolution_factor_sensor_anteile_caption"}}
   
		remote.call("EvoGUI", "create_remote_sensor", sensor_data)
		
		global.ist_registriert = true
	end
end

function summe_format(v)
	local s = string.format("%d", math.floor(v))
	local pos = string.len(s) % 3
	if pos == 0 then pos = 3 end
	return string.sub(s, 1, pos) .. string.gsub(string.sub(s, pos+1), "(...)", " %1")
end

function prozent_format(v)
    local whole_number = math.floor(v)
    local fractional_component = math.floor((v - whole_number) * 10)
	
	return string.format("%d.%d%%", whole_number, fractional_component)
end

script.on_event(defines.events.on_tick, pollutiontest)
script.on_event(defines.events.on_entity_died , spawner_death)